// Treiber fr Ansteuerung Datafox AE III mit RS485 oder RS232
// Sprache: MS Visual C++ 5.0
// 20.1.99 Keller Version 1.0
// 28.6.99 Keller Version 1.1
//		nderungen: schreiben, lesen und ndern der Konfiguration ergnzt
//					512kByte RAM im Geraet untersttzen.
//					Achtung! Der Treiber ab V1.1 laeuft nur mit Firmware V5.02 oder hoeher
//					Wartezeit eingefgt wenn Busnummer sich aendert. keine zusaetzliche Zeit notwendig
//

#define COM1	0		// Parameter von ComInit
#define COM2	1
#define COM3	2
#define COM4	3

#define COM_TIME_OUT 3000	// Wartezeit auf Zeichen von Geraet in ms

#define MAX_BUF 0x20000			// Buffer fr Daten aus Datafox
#define MAX_MB	300				// max. MOD-Bus Datenpaket

#define MAX_NETZ_GERAETE	31	// max. Gerte am Netz (ohne PC)

// Rckgabewerte von GetDatOnline und GetDatensatzAndClr
#define RET_DATA	1
#define RET_NO_DATA	0
#define RET_ERROR	2	// Fehler Kommunikation

// MOD-Bus Befehle
#define MBB_READ	1	// 128Byte RAM lesen
#define MBB_RD_PT	2	// Read/Write-Pointer lesen
#define MBB_SET_PT	3	// Read Pointer setzen
#define MBB_WRITE	4	// x Byte in RAM schreiben
#define MBB_CLR_DAT 5	// Daten im Datafox lschen Offline
#define MBB_SET_DT	6	// Datum  Uhrzeit setzen
#define MBB_RD_X	7	// x Byte RAM lesen
#define MBB_BOOT    8   // Bootloader starten 
#define MBB_SETK    9   // PZE Kartennummer hinzufgen 
#define MBB_CLRK    10  // PZE Kartennummer lschen 
#define MBB_CLRAK   11  // PZE alle Kartennummer lschen 
#define MBB_SETMK   12  /* PZE Werte fr Meldung (Zeitkonto) setzen */
#define MBB_CLRMK   13  /* PZE Werte fr Meldung (Zeitkonto) lschen */
#define MBB_DAT_OL  14  /* Datensatz Online abfragen */
#define MBB_MESS_OL 15  /* Message Online */

#define MB_ADD_O 254		// Mod-Bus-Adresse Offline

// Datafox
#define RAM_DF_END 0x77f00 // letzte Speicherzelle im Datafox RAM bei AE III mit 512kB
#define MEMD_START 0x8000		// Startadd. Daten im RAM bei AE III mit 512kByte
	// Offsets der Felder des Datensatzes
#define EV_OFFSET_KENNUNG	0
#define EV_OFFSET_DAY		1	// 2 stellen BCD
#define EV_OFFSET_MON		2
#define EV_OFFSET_YEAR		3
#define EV_OFFSET_STD		4
#define EV_OFFSET_MIN		5
#define EV_OFFSET_SEK		6
#define EV_OFFSET_DAUER		7	// long 4 Byte
#define EV_OFFSET_Z			11	// 4 x long = 4x4 = 16 Byte

// Datensatz
#define MAX_Z	6				// max. Z-Felder in einem Datensatz
#define DAT_LEN	(11 + MAX_Z*4)	// Lnge eines Datensatzes
struct TDatensatz
{
  unsigned char k;	// Kennung 1 bis 10 entspricht Tasten F1 bis F10; K. 14 ist Pause
  long date_time;	// Datum und Uhrzeit
  float dauer;		// Dauer der Taetigkeit in Minuten
  long z[MAX_Z];	// Zusatzfelder
};

// Config-Daten im Datofox Ae Format
#define C_STR_LEN 26 /* Config String max. 25 Zeichen + 1 fr Endekennzeichnung*/
#define Z_FELD_FL 25 /* letztes Byte im String z_bez[x][x] ist Flag fue Quelle */
#define Z_FL_T		0x01 /* Maske eingabe von Tastatur */
#define Z_FL_B		0x02 /* Maske eingabe von Barcodeleser */
#define Z_FL_C		0x04 /* Maske eingabe von Chip-Card */
#define MAX_S_FELD 4 /* Anzahl S-Felder */
#define MAX_KENN_P  11 /* 10 Kennungen und Pause */
#define MAX_KENN    10
#define LAST_KENN	17
#define KENN_PAUSE  14
#define MAX_KENN_FELD (LAST_KENN+1) /* gre Felder fr alle Kennungen */
#define MAX_Z       6 /* Anzahl Zusatzfelder */
#define AMD_NO      0   /* Auftragsbezug  -> kein Auftragsbezug */
#define AMD_AUFT    1   /*                -> Auftragsbezug */
#define AMD_AUFT_P  2   /*                -> Auftrags- und Personalbezug */
#define AMD_FLG_AN  0x01 /* Abmelden notwendig vor Neuanmeldung */
#define AMD_FLG_AN2 0x02 /* Abmelden notw. fr Ttigkeit ohne Auftragsbezug */
#define AMD_FLG_AE  0x04 /* Auto Ein nach Spannungsausfall */
#define MDE_NO      0x00 /* MDE-Mode  -> MDE aus */
#define MDE_STST    0x01 /*           -> Start Stop */
#define MDE_Z1      0x02 /*           -> E1 Zhler */
#define MDE_SUM     0x04 /*           -> Summenstckzhler */
#define MDE_Z2      0x08 /*           -> E2 Zhler */
#define MDE_E1_NUR  0x10 /*           -> nur E1 zugelassen (Taste gesperrt) */
#define MDE_E2_NUR  0x20 /*           -> nur E1 zugelassen (Taste gesperrt) */
#define PZE_ON		0x01 /* pze_md  Bit 0 = 1 -> PZE aktiviert */
#define PZE_CHKK	0x02 /*         Bit 1 = 1 -> Kartenummer berprfen */
#define PZE_SCHALT	7	 /* Anz. Umschaltpunkte fr PZE Betriebsart */
#define PZEMD_GRUND 0
#define PZEMD_KOMMEN 1
#define PZEMD_GEHEN	2
#define PZEMD_LASTK	0x0B
#define RESERVE		26
typedef struct  {
    unsigned short version;
    unsigned char netz_f; /* TRUE -> Netzwerk aktiv */
    unsigned char bus_num;  /* Gertenummer 1...31 */
    char s_bez[MAX_S_FELD][C_STR_LEN]; /* Bezeichnung der S-Felder */
    char s_inh[MAX_S_FELD][C_STR_LEN]; /* Inhalt der S-Felder */
    char ken_bez[MAX_KENN_P][C_STR_LEN]; /* Bezeichnung der Kennungen */
    char z_bez[MAX_KENN_P][MAX_Z][C_STR_LEN]; /* Bez. der Zusatzfelder */
							/* z_bez[x][x][25] -> Bitfeld fr Quelle bei Eingabe */
							/* Bit 0 -> Tastatur, Bit 1 -> Barcode, Bit 2 -> Chip-Card */
    unsigned char auftr_md;     /* AMD_NO, AMD_AUFT, AMD_AUFT_P */
    unsigned char max_ken;      /* Anzahl der gltigen Kennungen (F-Tasten) */
    unsigned char max_ken_a;    /* Anz. Kennungen fr Auftragsbezug (nur wenn auftr_md != AMD_NO) */
                            /* max_ken_a mu <= sein als max_ken */
                            /* z.B. max_ken = 8; max_ken_a = 6 */
                            /*   F1...F8 sind aktiv */
                            /*   F1...F6 automatische behandlung von Auftrgen*/
    unsigned char f_anmeld;     /* F_Taste fr Anmelden */
    unsigned char f_abmeld;     /* F-Taste fr Abmelden */
    unsigned char mde_md;       /* MDE-Mode Bitfeld MDE_NO,MDE_STST,...*/
    unsigned long mde_to;       /* MDE TimeOut */
    unsigned char mde_entp;     /* MDE Entprellzeit Eingnge E1, E2 */
    unsigned char mde_k_e1;     /* MDE Kennung (F-Taste) E1 */
    unsigned char mde_k_e2;     /* MDE Kennung (F-Taste) E2 */
    unsigned char muss_z[MAX_KENN_P];   /* Kann/Mu-Feld */
    unsigned long blk_dauer;    /* Blinkdauer der F-Tasten LED's */
    unsigned char par_max[MAX_KENN_FELD]; /* Belegung der Zusatzfelder */
	int		max_z[MAX_KENN_P][MAX_Z];
	int		min_z[MAX_KENN_P][MAX_Z];
	unsigned short pass;
	unsigned char mde_k_kl;		/* Kennung fr MDE-Klemmen (Rsten) */
	unsigned char auftr_zc;		/* Anzahl Z-Felder Autokopie (1...5) */
    unsigned char buchung[MAX_KENN];    /* Buchung ohne Zeitmessung */
    unsigned char amd_flags;    /* Flags Bit 0 -> Abmelden notwendig */
                                /*           1 -> Abmelden notwendig fr Ttigkeit ohne Auftragsbezug*/
								/*           2 -> Auto Ein nach Spannungsausfall */
	unsigned char pze_md;		/* Bitfeld Personalzeit		*/
								/* Bit 0 PZE_ON */
	unsigned char pze_bh[PZE_SCHALT]; /* PZE Betriebsart Bit 7..5, Schaltzeit Stunden Bit 5..0 */
	unsigned char pze_m[PZE_SCHALT]; /* PZE Schaltzeit Minuten */
	unsigned char pze_d[PZE_SCHALT]; /* PZE Schaltzeit Wochentag Bit 0 -> Mo., Bit 1 -> Di. ... */
	unsigned char pze_mk;		/* Kennung fr Abfrage Zeitkonto */
	unsigned char reserve[RESERVE];	/* Reserve fr Erweiterungen */
} TConfig;

class CCom
{
public:
	CCom();
	int ComInit(int iComNum);	// Schnittstelle oeffnen; 
								// return TRUE -> oeffnen erfogreich; FALSE -> Fehler 							
	void ComClose();			// Schnittstelle schliesen
	// Kommunikationsmetoden, sprechen direkt das Gert an
	int ChkDaten(int BusNum, int *cnt);	// Test ob neue Daten im Datafox vorhanden
								// return TRUE -> Kommunikation erfogreich
								//        FALSE -> keine Kommunikation
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// cnt -> Anzahl Datensaetze im Datafox
	int GetDatensatzAndClr(int BusNum,TDatensatz *dat); // ein Datensatz aus Datafox lesen
								// und im Datafox RAM lschen
								// return TRUE -> Kommunikation erfogreich
								//        FALSE -> keine Kom. oder keine Daten Vorhanden
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// *dat	-> gelesener Datensatz
	int SetKartennummer(int BusNum,unsigned int num);	// Kartenummer fr PZE hinzufgen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// num -> Kartenummer
								// return TRUE -> Kartenummer hinzugefgt
								//        FALSE -> Kartennummer schon vorhanden oder kein Speicherplatz
								//				   oder keine Komunikation
	int ClrKartennummer(int BusNum,unsigned int num);	// Kartenummer fr PZE lschen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// num -> Kartenummer
								// return TRUE -> Kartenummer gelscht
								//        FALSE -> Kartennummer nicht vorhaden oder keine Kommunikation
	int ClrAllKartennummer(int BusNum);	// alle Kartenummern fr PZE lschen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// return TRUE -> Kartenummern gelscht
								//        FALSE -> keine Kommunikation
	int SetDatumZeit(int BusNum,int Second,int Minute,int Hour,int Day,int Month,int Year); 
								// Datum+Zeit setzen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// Das Jahr wird 4stellig bergeben
								// return TRUE ->  Datum+Uhrzeit gesetzt
								//        FALSE -> keine Kommunikation
	int SetMessage(int BusNum,unsigned int KNum,int m1,int m2);// Message (Zeitkonto) setzen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// m1, m2 -> Message Wertebereich -9999 bis +9999
								// return TRUE ->  Message gesetzt
								//        FALSE -> keine Kommunikation oder Kartennummer nicht vorhanden
	int ClrMessage(int BusNum,unsigned int KNum);// Message (Zeitkonto) lschen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// return TRUE ->  Message gelscht
								//        FALSE -> keine Kommunikation oder Kartennummer nicht vorhanden
	int GetCfgDaten(int BusNum); // Konfiguration aus Datafox lesen
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// return TRUE ->  Konfiguration gelesen
								//        FALSE -> keine Kommunikation
	int SetCfgDaten(int num);	// Konfiguration in Datafox schreiben
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// return TRUE ->  Konfiguration geschrieben
								//        FALSE -> keine Kommunikation
	// Metoden um die Cfg-Daten zu ndern
	// es werden die Daten in der Struktur "Cfg" auf dem PC gendert.
	// Um die nderungen im Datafox AE III wirksam werden zu lassen, mssen sie mit "SetCfgDaten()"
	// in das Gert geladen werden. Dabei werden alle im Gert vorhanden Einstellungen berschrieben!
	// Um nur einzelne Konfigurationseinstellungen zu ndern, ist folgender Ablauf notwendig:
	//		- Konfig.-Daten mit "GetCfgDaten()" aus dem Gert lesen;
	//		- Konfiguration mit den nachfolgenden Metoden ndern;
	//		- Genderte Konfiguration mit "SetCfgDaten()" in das Gert zurckladen.
	void SetCfgAnzKennungen(int BusNum, int Anz); // Anzahl aktiver Kennungen F1 bis Fx
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// Anz -> Anzahl der Kennungen (1 ... 10)
	void SetCfgMessage(int BusNum,int Kennung,CString KennungTxt="Abfrage",CString MessTxt1="Zeitkonto",CString MessTxt2="Uhrlaubskonto"); 
								// Messagefunktion
								// aktivieren/deaktivieren
								// BusNum -> Busnummer des Datafox (1...31,MB_ADD_O)
								// Kennung -> Kennung (F-Taste) mit der die Message abgefragt wird (1..10)
								//			  Kennung 0 -> Message lschen
								// Die folgenden Parameter sind Optional
								// KennungTxt -> Text fr Kennung (Default: "Abfrage")
								// MessTxt1 -> Text fr Message 1 (Default: "Zeitkonto")
								// MessTxt2 -> Text fr Message 2 (Default: "Uhrlaubskonto")
								// Bitte beachten!
								//		Wenn Kennungen hizukommen oder wegfallen mu die Anzahl
								//		der Kennungen mit der Metode "SetCfgAnzKennungen()" angepasst werden
	// Online Metoden
	int GetDatOnline(int num,TDatensatz *dat);
								// Datensatz Online auslesen
								// num -> BusNummer
								// dat -> Datensatz (nur wenn Rckgabewert = 1)
								// return	0 -> kein Datensatz vorhanden oder Fehler
								//			1 -> Datensatz gelesen
	void SendMess(int num, int ErrFlag, char *mes);
								// Message zum Datafox senden
								// ErrFlag	-> 1 -> Summer signalisiert Fehler
								//			-> 0 -> Summer sig. OK
protected:
	HANDLE hcom;	// Handle serielle Schnittst.
	int ComOpen;	// TRUE-> serielle Schnittst. geffnet
	unsigned char DfBuf[MAX_BUF];	// Buffer fr RAM-Daten Datafox
	unsigned char RBuf[MAX_MB];		// MOD-Bus Read-Buffer
	unsigned char WBuf[MAX_MB];		// MOD-Bus Write-Buffer
	unsigned char BusNumAkt;	// aktuelle Busnummer	
	TConfig Cfg[MAX_NETZ_GERAETE+1];	// Config-Daten von 31 Netzgerten Index 0..30 
										// und einem Offline-Gert Index 31

	int Command(unsigned char *wbuf,int WCnt,unsigned char *rbuf,int RCnt);
	int GetDfPt(int num, unsigned *ReadPt,unsigned *WritePt,unsigned short *ConfigPt,unsigned short *LstPt);
	void MkMBPaket(unsigned char *buf,int cnt);	// CRC berechnen
	int CheckMBPaket(unsigned char *buf, int cnt);
	unsigned short int BinToUInt(unsigned char *buf);
	unsigned long BinToULong(unsigned char *buf);
	int GetRamDf(int num, unsigned int StartAdd, unsigned int EndAdd,HWND Wnd,int Progress);
	int GetAlleDaten(int AktNum,unsigned int ReadPt, unsigned int WritePt, HWND Wnd, int mess);
	void ULongToBin(unsigned char *buf,unsigned long val);
	int GetXByte(int BusNum,unsigned add, int cnt, unsigned char *buf);
	int BcdToInt(unsigned char val);
	int SetReadPt(int BusNum,unsigned int ReadPt);
	unsigned char IntTo2BCD(int num);
	void IntToBin(unsigned char *buf,int val);
	void SetCfgDefault();	// Vorgabewerte der Config-Daten setzen
	long LongIntelToMotorola(long val);
	int SetRamDf(int num, unsigned short StartAdd,int cnt,unsigned char *buf);
	void UShortToBin(unsigned char *buf,unsigned long val);
};